﻿using System;
using System.Data.Sql;
using System.Data.SqlTypes;
using System.Runtime.Serialization;
using Microsoft.SqlServer.Server;

[Serializable()]
[SqlUserDefinedType(Format.Native,
	ValidationMethodName = "ValidateComplex")]
public struct Complex : INullable
{

	private bool is_Null;
	private double _real;
	private double _imaginary;

	// Track whether this instance is a null
	public bool IsNull
	{
		get
		{
			return is_Null;
		}
	}

	// Return a string representation
	public override string ToString()
	{
		if (this.is_Null)
			return "NULL";
		else
			return _real + ", " + _imaginary;
	}

	// Convert string representation to an instance
	[SqlMethod(OnNullCall = false)]
	public static Complex Parse(SqlString s)
	{
		if (s.IsNull)
			return Null;
		else
		{
			// Parse input string here to 
			// separate out parts
			Complex c = new Complex();
			string str = Convert.ToString(s);
			string[] parts = str.Split(',');
			c.Real = Convert.ToDouble(parts[0]);
			c.Imaginary = Convert.ToDouble(parts[1]);
			// Check for valid input
			if (!c.ValidateComplex())
				throw new ArgumentException(
				   "Invalid coordinate values.");
			return c;
		}
	}

	// Return a null instance
	public static Complex Null
	{
		get
		{
			Complex c = new Complex();
			c.is_Null = true;
			return c;
		}
	}

	// Real part
	public double Real
	{
		get
		{
			return (this._real);
		}
		set
		{
			this._real = value;
		}
	}

	// Imaginary part
	public double Imaginary
	{
		get
		{
			return (this._imaginary);
		}
		set
		{
			this._imaginary = value;
		}
	}

	// Validation method to enforce valid real 
	// and complex values.
	private bool ValidateComplex()
	{
		// All values of real and 
		// imaginary parts are valid.
		// This method is included 
		// just to show the mechanics
		return true;
	}

	// Add this instance to another
	[SqlMethod]
	public Complex AddTo(Complex c)
	{
		Complex res = new Complex();
		res.Real = this._real + c.Real;
		res.Imaginary = this._imaginary + c.Imaginary;
		return res;
	}
}
